/*
 * mygpio: is a program that re-implements the wiringPi's gpio comand
 * it takes the following parameters:
 * either "read" or "write"
 * the pin number (0..7), the logic level
 * copyright U. Raich 2017
 * This program is part of the workshop on embedded systems held at 
 * the University of Cape Coast in the winter semester 2017
 * It has been released under the GNU General Public License
 * see gnu.org/licenses/gpl.html
 */

#include <stdio.h>
#include <wiringPi.h>
#include <string.h>
#include <stdlib.h>

void usage(char *cmd) {
  printf("Usage: %s write pin number logic level\n",cmd);
  printf(" or  : %s read  pin number\n",cmd);
}

int main(int argc, char **argv) {
  char *readString = "read";
  char *writeString = "write";
  long pinNumber=1,value=1;
  char *endPtr;
  
  if (argc < 3) {
    usage(argv[0]);
    return(.1);
  }
  
  /* setup the library */
  wiringPiSetup () ;  
  /* 
     try to convert the pin number
   */
  pinNumber = strtol(argv[2],&endPtr,10);
  if ((pinNumber == 0) && (endPtr == argv[2])) {
    printf("Could not convert pin number! %s should be 0 .. 7)\n",argv[2]);
    exit(-1);
  }
  else {
    if ((pinNumber < 0) || (pinNumber > 7)) {
      printf("Pin numberis illegal! %s should be 0 .. 7)\n",argv[2]);
      exit(-1);
    }     
    printf("Pin number is %d\n", (int)pinNumber);
  }
  /*
    check if we have to read
  */
  if (!strcmp(readString,argv[1])) {
      printf("read cmd\n");
      if (argc !=3 ) {
	usage(argv[0]);
	return (-1);
      }
      pinMode(pinNumber,INPUT);
      if (digitalRead(pinNumber))
	printf("Pin %d is high\n",(int)pinNumber);
      else
	printf("Pin %d is low\n",(int)pinNumber);
	
    }
  /*
    check if we have to write
  */
  else if (!strcmp(writeString,argv[1])) {
      printf("write cmd\n");

      if (argc != 4) {
	usage(argv[0]);
	return (-1);
      }
      
      value = strtol(argv[3],&endPtr,10);
      
      if ((value == 0) && (endPtr == argv[3])) {
	printf("Could not convert logic value: %s\n",argv[3]);
	exit(-1);
      }
      else
	printf("Logic level is %d\n",(int)value);
      if ((value < 0) || (value > 1)) {
	printf("%s is not a logic level\n",argv[3]);
	exit(-1);
      }
      printf("Writing %d to %d\n",(int)value,(int)pinNumber);
      digitalWrite((int)pinNumber,(int) value);
  }
  else
    usage(argv[0]);
  return 0;
}
